#!/usr/bin/env python
# -*- coding: utf-8 -*-
# File  : get_data.py
# Author: My Duan
# Date  : 2022-09-30

import pandas as pd
from data_loader import *
from survival_utils import _make_R
from options import parse_args

def match_survival_data(obj_data, survival_df):

	survival_samples = [i for i in list(obj_data.index) if i in list(survival_df.index)]

	survival_info = survival_df.loc[survival_samples, ['OS', 'OS.time']]
	survival_info['censored'] = survival_info.OS.apply(lambda x: 0 if x==1 else 1)
	survival_info['survival'] = survival_info['OS.time']
	temp_data = pd.merge(obj_data, survival_info, left_index=True, right_index=True, how='left')
	temp_data = temp_data.dropna(subset=['censored','survival']) # how='any

	obj_data = temp_data.iloc[:,:-2]
	survival_label = temp_data.loc[:,['censored','survival']]

	return obj_data, survival_label

def get_split_df(clinical_day):

	day = np.argsort(clinical_day)
	day_stratified = np.zeros(day.shape[0])
	days = len(day) / 5
	day_stratified[day < days] = 0
	day_stratified[(day >= days) & (day < 2 * days)] = 1
	day_stratified[(day >= 2 * days) & (day < 3 * days)] = 2
	day_stratified[(day >= 3 * days) & (day < 4 * days)] = 3
	day_stratified[day >= 4 * days] = 4

	return day_stratified


def preprocess_feats(obj_data, regu, target):

	# fill all the missing features and unavaliable features by 0
	# transform omit: feats
	# if exits, extract as regu/targ, else fill by 0
	feats = obj_data.columns.to_list()
	feats = list(np.unique(np.array(feats)))

	if feats[0].find('.') != -1:
		feats = [i.split('.')[0] for i in feats]

	obj_data.columns = feats

	unavaliable_feats = list(np.array(feats)[obj_data.isnull().all().values])

	for k in regu:
		if k not in feats:
			unavaliable_feats.append(k)

	for k in target:
		if k not in feats:
			unavaliable_feats.append(k)

	# print('Here')
	# print(len(unavaliable_feats))

	if len(unavaliable_feats) !=0:
		print('\tUnavaliable features number is {0}. Padding these genes by 0.0.'.format(len(unavaliable_feats)))
		obj_data.loc[:, unavaliable_feats] = 0.0

	else:
		print('\tAll regulatory and target genes are avaliable.\n')

	return  obj_data


def split_data(obj_data, survival_label, test_size=0.2):

	day_stratified = get_split_df(survival_label['survival'].values)
	survival_label['stratified'] = day_stratified

	print('Dataset censord label distribution is :')
	print(survival_label.censored.value_counts())

	print('Split dataset 8:1:1')
	from sklearn.model_selection import train_test_split
	train_x, rest_x, train_y, rest_y = train_test_split(obj_data, survival_label,
														test_size=test_size,
														stratify=survival_label.stratified)

	test_x, valid_x, test_y, valid_y = train_test_split(rest_x, rest_y,
														test_size=0.5,
														stratify=rest_y.stratified)

	print('KNN imputation...')
	from sklearn.impute import KNNImputer
	imputer = KNNImputer()
	imputer.fit(train_x)
	train_x_impute = imputer.transform(train_x)
	test_x_impute = imputer.transform(test_x)
	valid_x_impute = imputer.transform(valid_x)

	print('Z-normlization...')
	from sklearn.preprocessing import StandardScaler
	std = StandardScaler()
	std.fit(train_x_impute)
	train_x_std = std.transform(train_x_impute)
	test_x_std  = std.transform(test_x_impute)
	valid_x_std = std.transform(valid_x_impute)

	'''transform the array to dataframe'''
	train_df = pd.DataFrame(train_x_std, index=train_x.index.to_list(), columns=train_x.columns.to_list())
	test_df  = pd.DataFrame(test_x_std, index=test_x.index.to_list(), columns=test_x.columns.to_list())
	valid_df = pd.DataFrame(valid_x_std, index=valid_x.index.to_list(), columns=valid_x.columns.to_list())

	return train_df, test_df, valid_df, train_y, test_y, valid_y


def get_adjency(adj_matrix):

	edge = np.where(abs(adj_matrix) != 0)
	edge_index = np.vstack((edge[0], edge[1]))

	return edge_index


def get_loader(*array, edge_index):

	opt = parse_args()

	train_driver, test_driver, valid_driver, train_targ, test_targ, valid_targ, train_y, test_y, valid_y = array

	train_dl = get_survival_loader(train_driver, train_targ, train_y, edge_index, batch_size=opt.batch_size, drop_last=True)
	valid_dl = get_survival_loader(valid_driver, valid_targ, valid_y, edge_index, batch_size=1000, drop_last=False)
	test_dl =  get_survival_loader(test_driver, test_targ, test_y, edge_index, batch_size=1000, drop_last=False)

	tr_R = _make_R(train_y.survival.values)
	va_R = _make_R(valid_y.survival.values)
	te_R = _make_R(test_y.survival.values)

	input_dim = train_driver.shape[1]

	return train_dl, valid_dl, test_dl, tr_R, va_R, te_R, input_dim



